#include "QUANTA/QUANTAinit.hxx"
#include "QUANTA/QUANTAnet_blockingTcp_c.h"
#include "QUANTA/QUANTAts_thread_c.hxx"

#include <stdio.h>
#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif
#include <stdlib.h>

/* This is the size of all messages sent from clients to servers. */
#define TRANSMITTED_DATA_SIZE 512
#define APPLICATION_LAUNCH_PORT 32392

static void Process();

static QUANTAnet_tcpClient_c *client = NULL;
static char clientData[TRANSMITTED_DATA_SIZE] = "";
static QUANTAnet_blockingTcpServer_c *server;
static QUANTAts_thread_c QUANTAThread;

static bool IsAbsoluatePath(char *fileName)
{
	if (fileName) {
		return (strlen(fileName) > 2 && fileName[1] == ':' && (fileName[2] == '\\' || fileName[2] == '/'));
	}
	return false;
}

static void ExecuteAnApplication(char *app)
{
#ifdef WIN32
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    TCHAR szCmdLine[MAX_PATH * 4] = "";

    if (GetEnvironmentVariable("COMSPEC", szCmdLine, MAX_PATH)) {
        strcat(szCmdLine, " /C ");
	}
	if (*szCmdLine) 
		strcat(szCmdLine, " start ");
	else 
		strcpy(szCmdLine, " start ");
	strcat(szCmdLine, "\"\"\ \"");
	strcat(szCmdLine, app);
	strcat(szCmdLine, "\"");

    ZeroMemory(&si, sizeof (STARTUPINFO));
    si.cb = sizeof (STARTUPINFO);
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_SHOWNORMAL;

	char curDir[MAX_PATH] = "";
	if (IsAbsoluatePath(app)) {
		char drive[_MAX_DRIVE];
		char dir[_MAX_DIR];
		char fname[_MAX_FNAME];
		char ext[_MAX_EXT];

		_splitpath(app, drive, dir, fname, ext);
		strcpy(curDir, drive);
		strcat(curDir, dir);

	}
    if (CreateProcess(NULL, szCmdLine, NULL, NULL, FALSE, 
		NORMAL_PRIORITY_CLASS, NULL, (curDir[0] == '\0') ? NULL : curDir, &si, &pi)) { 
		HANDLE hProcess = pi.hProcess;

		CloseHandle(pi.hThread);
	}
#else
	system(app);
#endif
}

static void TrimEndOfBuffer(char* buffer)
{
	for (int i = strlen(buffer) - 1; i >= 0; i--) {
		if (isspace(buffer[i]))
			buffer[i] = '\0';
		else
			break;
	}
}

static void *ExecuteApplicationThread(void *param)
{
	QUANTAnet_tcpClient_c *client = (QUANTAnet_tcpClient_c *) param;
	char buffer[TRANSMITTED_DATA_SIZE];
	int size = TRANSMITTED_DATA_SIZE;

	if (client) {
		int status = client->read(buffer, &size, QUANTAnet_tcpClient_c::BLOCKING);
		while (status == QUANTAnet_tcpClient_c::OK) {
			printf("RECEIVED: %s\n", buffer);
			TrimEndOfBuffer(buffer);
			ExecuteAnApplication(buffer);
			status = client->read(buffer, &size, QUANTAnet_tcpClient_c::BLOCKING);
		}
	}

	delete client;
	return NULL;
}

static void Process()
{
	QUANTAnet_tcpClient_c *aClient;

	while(1) {
		/* If a connection is established, accept it and create
		 * a new socket for that connection to communicate with.
		 * Then, waits for a command from a client, execute the command,
		 * After the execution, it closes the connection and waits for a new connection.
		 */
		aClient = server->checkForNewConnections();
		if (aClient) 
			QUANTAThread.create(ExecuteApplicationThread, aClient);
		else {
			printf("error occurred while waiting for a connection.\n");
			break;
		}
	}
}

static void CloseDownSockets()
{
	server->close();
	delete server;
}

int main(int argc, char** argv)
{
	QUANTAinit();

	int port = APPLICATION_LAUNCH_PORT;
	server = new QUANTAnet_blockingTcpServer_c;
	if (server->init(port) == 0) {
		exit(1);
	}
	printf("waiting for a connection in %d\n", APPLICATION_LAUNCH_PORT);

	void Process();
	Process();

	return 0;
}