#ifdef _WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#if _MSC_VER >= 1400
#include <fcntl.h>
#endif
//#include "char.h"
#include "profile.h"

#define MAX_LEN 3000

// C++ version of strdup function
char *strDup(char *str)
{
	char *tempstr = new char[strlen(str) + 1];
	if (tempstr) strcpy(tempstr, str);
	return tempstr;
}

void MemMove(char *dest, char *orig, int len)
{
	for (int i = 0; i < len; i++) {
		dest[i] = orig[i];
	}
}

char *trimLeadingSpaces(char *str)
{
	size_t index = 0;
	size_t len = strlen(str);

	if (len > 0) {
		while (index < len && str[index] == ' ' || str[index] == '\t' || str[index] == '\n' || str[index] == '\r')
			index++;

		if (index > 0) {
			memmove(str, str + index, sizeof(char) * (len - index) + 1);
		}
	}
	return str;
}

char *trimEndingSpaces(char *str)
{
	size_t len = strlen(str);
	size_t index = len - 1;

	if (len > 0) {
		while (index > 0 && str[index] == ' ' || str[index] == '\t' || str[index] == '\n' || str[index] == '\r')
			index--;
		
		if (index > 0) str[index + 1] = '\0';
	}
	return str;
}

char* trim(char *str) 
{
	trimLeadingSpaces(str);
	trimEndingSpaces(str);
	return str;
}

void trimLeadingChar(char *str, char ch)
{
	if (str && str[0] == ch) memmove(str, str + 1, sizeof(char) * strlen(str));
}

void trimEndingChar(char *str, char ch)
{
	if (str) {
		size_t len = strlen(str);
		if (str[len - 1] == ch) str[len - 1] = '\0';
	}
}

// Trims leading and ending specific character
void trimChar(char *str, char ch) 
{
	size_t len = strlen(str);
	if (str[0] == ch) {
		memmove(str, str + 1, sizeof(char) * len);
		len--;
	}

	if (str && str[len - 1] == ch) str[len - 1] = '\0';
}


int indexOf(char *str, char ch)
{
	size_t len = strlen(str);

	for (int i = 0; i < len; i++) {
		if (str[i] == ch) return i;
	}
	return -1;
}
	
char *substring2(char *str, int start, int last)
{
	if (start > last) return NULL;
	size_t len = strlen(str);
	size_t l = (len > last) ? last : len;

	if (start < 0) start = 0;
	char *dest = new char[last - start + 1];
	if (dest) {
		strncpy(dest, str + start, last - start);
		dest[last - start] = '\0';
		return dest;
	}
	return NULL;
}

char *substring(char *str, int start, int last)
{
	if (start > last) return NULL;
	size_t len = strlen(str);
	size_t l = (len > last) ? last : len;

	if (start < 0) start = 0;
	char *dest = new char[last - start + 1];
	if (dest) {
		strncpy(dest, str + start, last - start);
		dest[last - start] = '\0';
		strcpy(str, dest);
		delete dest;
		return str;
	}
	else return NULL;
}

char *getPrivateProfileString(char *sn, char *kn,
							   char *ds, char *fn) 
{
	FILE *raf;
	char str[MAX_LEN];
	char *tempstr = NULL;
	char *temp = NULL;
	int no;
	int eof = false;
	
#if _MSC_VER >= 1400
	_set_fmode(_O_TEXT);
#endif
	raf = fopen(fn, "rt");
#if _MSC_VER >= 1400
	_set_fmode(_O_BINARY);
#endif
	if (raf == NULL) {
#ifdef _DEBUG
		printf("Could not find a file called %s\n", fn);
#endif
		return strDup(ds);
	}
	if (fgets(str, MAX_LEN, raf) == NULL) {
		fclose(raf);
		return strDup(ds);
	}

	while (!feof(raf) && str[0]) {
		tempstr = strDup(str);
		if (tempstr == NULL) {
#ifdef _DEBUG
			printf("Out of memory\n");
#endif
			break;
		}

		trim(tempstr);
            
        // ignore comments
        no = indexOf(tempstr, '#');
        if (no != -1) tempstr[no] = '\0';

        no = indexOf(tempstr, ';');
        if (no != -1) tempstr[no] = '\0';
        
		if(strlen(tempstr) > 0 && (tempstr[0] == '[') && (tempstr[strlen(tempstr) - 1] == ']')) {
			trimLeadingChar(tempstr, '[');
			trimEndingChar(tempstr, ']');

			if (_stricmp(tempstr, sn) == 0) {
				while (!feof(raf) && fgets(str, MAX_LEN, raf)) {
					delete tempstr;
					tempstr = strDup(str);
					if (tempstr == NULL) {
#ifdef _DEBUG
						printf("Out of memory.\n");
#endif
						fclose(raf);
						return strDup(ds);
					}
                    trim(tempstr);

					if(strlen(tempstr) > 0) {
						if (tempstr[0] != '[') {
							// ignore comments
							no = indexOf(tempstr, '#');
							if (no != -1) tempstr[no] = '\0';

							no = indexOf(tempstr, ';');
							if (no != -1) tempstr[no] = '\0';
                            
							if (strlen(tempstr) > 0) {
      							no = indexOf(tempstr, '=');
								tempstr[no] = '\0';
								
								char *tempKeyWord = strDup(tempstr);
								if (tempKeyWord) {
									trim(tempKeyWord); 
			    					if(_stricmp(tempKeyWord, kn) == 0) {
										char *tempValue = strDup(tempstr + no + 1);
										if (tempValue == NULL) {
											delete tempKeyWord;
											delete tempstr;
											return strDup(ds);
										}
										trim(tempValue);
#ifdef _DEBUG
										printf("found %s\n", tempValue);
#endif
										fclose(raf);
										delete tempstr;
										delete tempKeyWord;
										trimChar(tempValue, '"');
										return tempValue; // found
									}
									else delete tempKeyWord;
								}
    						}
						}
						else { // not found
							fclose(raf);
							delete tempstr;
							return strDup(ds);
						}
					} 					
				} // while
			}
		} 
#ifdef _DEBUG
		else {
			printf("not a section\n");
		}
#endif
		delete tempstr;
		if (fgets(str, MAX_LEN, raf) == NULL) break;
	} // while str!=null

	fclose(raf);
	return strDup(ds);
}

int getPrivateProfileInt(char *sn, char *kn, int data, char *fn)
{
	char s[30];

	sprintf(s, "%d", data);
	char *temp = getPrivateProfileString(sn, kn, s, fn);

	if (temp) {
		int result = atoi(temp);
		delete temp;
		return result;
	}
	else return data;
}

float getPrivateProfileFloat(char *sn, char *kn, float data, char *fn)
{
	char s[30];
	sprintf(s, "%f", data);
	char *temp = getPrivateProfileString(sn, kn, s, fn);

	if (temp) {
		float result = (float) atof(temp);
		delete temp;
		return result;
	}
	else return data;
}

/*void main()
{
	char t[100] = "    HE#LLO HOW ARE YOU           ?          ";

	int len = strlen(t);
	printf("len = %d original = %s\n", len, t);
	trimLeadingSpaces(t);
	len = strlen(t);
	printf("len = %d 1        = %s\n", len, t);
	trimEndingSpaces(t);
	len = strlen(t);
	printf("len = %d 2        = %s\n", len, t);

	char *newTemp;

	int i = indexOf(t, '#');
	if (i != -1) {
		newTemp = substring(t, 0, i);
		printf("newTemp = %s\n", newTemp);
	}
	i = indexOf(newTemp, '#');
	if (i != -1) {
		newTemp = substring(newTemp, 0, i);
		printf("newTemp = %s\n", newTemp);
	}
	else {
		printf("NO #.\n");
	}

	char *temp = getPrivateProfileString("temp", "hello", 
							   "win32", "temp.ini");
	if (temp) {
		printf("temp = %s\n", temp);
		delete temp;
	}

	temp = getPrivateProfileString("temp", "hello2", 
							   "win32", "temp.ini");
	if (temp) {
		printf("temp = %s\n", temp);
		delete temp;
	}

	temp = getPrivateProfileString("temp2", "hello", 
							   "win32", "temp.ini");
	if (temp) {
		printf("temp = %s\n", temp);
		delete temp;
	}
	int a = getPrivateProfileInt("cavebuttons", "cavebutton1", 0, "trackd_joystick.ini");
	printf("temp = %d\n", a);
}*/

